"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OcsError = exports.ConflictError = exports.IncorrectPathTypeError = exports.NotReadyError = exports.NotFoundError = exports.ForbiddenError = exports.Exception = void 0;
function createErrorType(initialize = undefined, ErrorClass = undefined, prototype = undefined) {
    ErrorClass = ErrorClass || Error;
    let Constructor = function (message) {
        let error = Object.create(Constructor.prototype);
        error.message = message;
        error.stack = (new Error).stack;
        if (initialize) {
            initialize(error, message);
        }
        return error;
    };
    Constructor.prototype = Object.assign(Object.create(ErrorClass.prototype), prototype);
    return Constructor;
}
exports.Exception = createErrorType();
exports.ForbiddenError = createErrorType(function forbiddenErrorConstructor(error, path) {
    error.message = `Access to ${path} was denied`;
}, exports.Exception);
exports.NotFoundError = createErrorType(function notFoundErrorConstructor(error, path) {
    error.message = `${path} not found!`;
}, exports.Exception);
exports.NotReadyError = createErrorType(function notReadyErrorConstructor(error) {
    error.message = 'The Nextcloud instance is initializing…';
}, exports.Exception);
exports.IncorrectPathTypeError = createErrorType(function incorrectPathTypeErrorConstructor(error, { path, type }) {
    error.message = `The path '${path}' is not a ${type}`;
}, exports.Exception);
exports.ConflictError = createErrorType(function conflictErrorConstructor(error, path) {
    error.message = `Conflict on ${path}`;
}, exports.Exception);
exports.OcsError = createErrorType(function ocsErrorConstructor(error, { message, identifier, reason, statusCode }) {
    const id = (identifier ? ` '${identifier}'` : '');
    error.name = 'OcsError';
    error.message = `${message}${id}: ${reason}`;
    if (statusCode) {
        error.statusCode = statusCode;
    }
}, exports.Exception);
