var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { promisify, rejectWithOcsError } from './helper';
import { ocsGetActivities } from './activity';
import { OcsConnection } from './ocs-connection';
import { OcsSharePermissions, } from './types';
import { ocsResendUserWelcomeEmail, ocsAddRemoveUserForGroup, ocsGetUserSubAdmins, ocsSetUserSubAdmin, ocsSetUserEnabled, ocsGetUserGroups, ocsDeleteUser, ocsListUsers, ocsEditUser, ocsAddUser, ocsGetUser, } from './user';
import { ocsGetGroupSubAdmins, ocsGetGroupUsers, ocsDeleteGroup, ocsListGroups, ocsAddGroup, } from './group';
import { ocsDeleteShare, ocsEditShare, ocsGetShares, ocsAddShare, ocsGetShare, } from './share';
import { ocsGetGroupfolders, ocsGetGroupfolder, ocsAddGroupfolder, ocsRemoveGroupfolder, ocsAddGroupfolderGroup, ocsRemoveGroupfolderGroup, ocsEnableOrDisableGroupfolderACL, ocsRenameGroupfolder, ocsSetGroupfolderQuota, ocsSetGroupfolderPermissions, ocsSetGroupfolderManageACL, } from './groupfolders';
const promisifiedGetActivities = promisify(ocsGetActivities);
const promisifiedResendUserWelcomeEmail = promisify(ocsResendUserWelcomeEmail);
const promisifiedAddRemoveUserForGroup = promisify(ocsAddRemoveUserForGroup);
const promisifiedGetUserSubAdmins = promisify(ocsGetUserSubAdmins);
const promisifiedSetUserSubAdmin = promisify(ocsSetUserSubAdmin);
const promisifiedSetUserEnabled = promisify(ocsSetUserEnabled);
const promisifiedGetUserGroups = promisify(ocsGetUserGroups);
const promisifiedDeleteUser = promisify(ocsDeleteUser);
const promisifiedListUsers = promisify(ocsListUsers);
const promisifiedEditUser = promisify(ocsEditUser);
const promisifiedAddUser = promisify(ocsAddUser);
const promisifiedGetUser = promisify(ocsGetUser);
const promisifiedGetGroupSubAdmins = promisify(ocsGetGroupSubAdmins);
const promisifiedGetGroupUsers = promisify(ocsGetGroupUsers);
const promisifiedDeleteGroup = promisify(ocsDeleteGroup);
const promisifiedListGroups = promisify(ocsListGroups);
const promisifiedAddGroup = promisify(ocsAddGroup);
const promisifiedDeleteShare = promisify(ocsDeleteShare);
const promisifiedEditShare = promisify(ocsEditShare);
const promisifiedGetShares = promisify(ocsGetShares);
const promisifiedGetShare = promisify(ocsGetShare);
const promisifiedAddShare = promisify(ocsAddShare);
const promisifiedGetGroupfolders = promisify(ocsGetGroupfolders);
const promisifiedGetGroupfolder = promisify(ocsGetGroupfolder);
const promisifiedAddGroupfolder = promisify(ocsAddGroupfolder);
const promisifiedRemoveGroupfolder = promisify(ocsRemoveGroupfolder);
const promisifiedAddGroupfolderGroup = promisify(ocsAddGroupfolderGroup);
const promisifiedRemoveGroupfolderGroup = promisify(ocsRemoveGroupfolderGroup);
const promisifiedEnableOrDisableGroupfolderACL = promisify(ocsEnableOrDisableGroupfolderACL);
const promisifiedRenameGroupfolder = promisify(ocsRenameGroupfolder);
const promisifiedSetGroupfolderQuota = promisify(ocsSetGroupfolderQuota);
const promisifiedSetGroupfolderPermissions = promisify(ocsSetGroupfolderPermissions);
const promisifiedSetGroupfolderManageACL = promisify(ocsSetGroupfolderManageACL);
export function configureOcsConnection(options) {
    const self = this;
    self.ocsConnection = new OcsConnection({
        url: options.url,
        username: options.username,
        password: options.password
    });
}
export function getActivities(connection, fileId, sort, limit, sinceActivityId) {
    return __awaiter(this, void 0, void 0, function* () {
        let activities;
        try {
            activities = yield promisifiedGetActivities.call(connection, (typeof fileId === 'string' ? parseInt(fileId, 10) : fileId), sort || 'desc', limit || -1, sinceActivityId || -1);
        }
        catch (error) {
            activities = rejectWithOcsError(error, {
                message: 'Unable to get activities for',
                identifier: fileId,
                useMeta: false,
                customErrors: {
                    [204]: 'The user has selected no activities to be listed in the stream',
                    [304]: 'ETag/If-None-Match are the same or the end of the activity list was reached',
                    [403]: 'The offset activity belongs to a different user or the user is not logged in',
                    [404]: 'The filter is unknown'
                }
            });
        }
        return activities;
    });
}
export function getUser(connection, userId) {
    return __awaiter(this, void 0, void 0, function* () {
        let user;
        try {
            user = yield promisifiedGetUser.call(connection, userId);
        }
        catch (error) {
            user = rejectWithOcsError(error, {
                message: 'Unable to find user',
                identifier: userId,
                useMeta: false
            });
        }
        return user;
    });
}
export function setUserEnabled(connection, userId, isEnabled) {
    return __awaiter(this, void 0, void 0, function* () {
        let success;
        try {
            success = yield promisifiedSetUserEnabled.call(connection, userId, isEnabled);
        }
        catch (error) {
            success = rejectWithOcsError(error, {
                message: `Unable to ${isEnabled ? 'enable' : 'disable'} user`,
                identifier: userId,
                useMeta: true,
                customErrors: {
                    [101]: 'user does not exist'
                }
            });
        }
        return success;
    });
}
export function editUser(connection, userId, field, value) {
    return __awaiter(this, void 0, void 0, function* () {
        let userEdited;
        try {
            userEdited = yield promisifiedEditUser.call(connection, userId, field, value);
        }
        catch (error) {
            userEdited = rejectWithOcsError(error, {
                message: 'Unable to edit user',
                identifier: userId,
                useMeta: true,
                expectedErrorCodes: [400, 401],
                customErrors: {
                    [101]: 'user not found',
                    [997]: 'possible reasons: Does it exist? Do you have the right permissions? Is the field valid?'
                }
            });
        }
        return userEdited;
    });
}
export function getUserGroups(connection, userId) {
    return __awaiter(this, void 0, void 0, function* () {
        let groups;
        try {
            groups = yield promisifiedGetUserGroups.call(connection, userId);
        }
        catch (error) {
            groups = rejectWithOcsError(error, {
                message: 'Unable to get groups for user',
                identifier: userId,
                useMeta: false
            });
        }
        return groups;
    });
}
export function getUserSubAdmins(connection, userId) {
    return __awaiter(this, void 0, void 0, function* () {
        let subAdmins;
        try {
            subAdmins = yield promisifiedGetUserSubAdmins.call(connection, userId);
        }
        catch (error) {
            subAdmins = rejectWithOcsError(error, {
                message: 'Unable to get sub-admins for user',
                identifier: userId,
                useMeta: true,
                expectedErrorCodes: [400],
                customErrors: {
                    [101]: 'user does not exist'
                }
            });
        }
        return subAdmins;
    });
}
export function resendUserWelcomeEmail(connection, userId) {
    return __awaiter(this, void 0, void 0, function* () {
        let success;
        try {
            success = yield promisifiedResendUserWelcomeEmail.call(connection, userId);
        }
        catch (error) {
            success = rejectWithOcsError(error, {
                message: 'Unable to resend welcome email for user',
                identifier: userId,
                useMeta: true,
                expectedErrorCodes: [400],
                customErrors: {
                    [101]: 'email address not available',
                    [102]: 'sending email failed'
                }
            });
        }
        return success;
    });
}
export function addRemoveUserForGroup(connection, userId, groupId, toAdd) {
    return __awaiter(this, void 0, void 0, function* () {
        let userModifiedForGroup;
        try {
            userModifiedForGroup = yield promisifiedAddRemoveUserForGroup.call(connection, userId, groupId, toAdd);
        }
        catch (error) {
            userModifiedForGroup = rejectWithOcsError(error, {
                message: `Unable to ${toAdd ? 'add' : 'remove'} user '${userId}' ${toAdd ? 'to' : 'from'} group`,
                identifier: groupId,
                useMeta: true,
                expectedErrorCodes: [400],
                customErrors: {
                    [101]: 'no group specified',
                    [102]: 'group does not exist',
                    [103]: 'user does not exist',
                    [104]: 'insufficient privileges',
                }
            });
        }
        return userModifiedForGroup;
    });
}
export function addRemoveUserSubAdminForGroup(connection, userId, groupId, toAdd) {
    return __awaiter(this, void 0, void 0, function* () {
        let subAdminModifiedForGroup;
        try {
            subAdminModifiedForGroup = yield promisifiedSetUserSubAdmin.call(connection, userId, groupId, toAdd);
        }
        catch (error) {
            let customErrors = {};
            if (toAdd) {
                customErrors[101] = 'user does not exist';
                customErrors[102] = 'group does not exist';
            }
            else {
                customErrors[101] = 'user or group does not exist';
                customErrors[102] = 'user is not a sub-admin of the group';
            }
            subAdminModifiedForGroup = rejectWithOcsError(error, {
                customErrors,
                message: `Unable to ${toAdd ? 'add' : 'remove'} user '${userId}' as sub-admin ${toAdd ? 'to' : 'from'} group`,
                identifier: groupId,
                useMeta: true,
                expectedErrorCodes: [400],
            });
        }
        return subAdminModifiedForGroup;
    });
}
export function listUsers(connection, search, limit, offset) {
    return __awaiter(this, void 0, void 0, function* () {
        let users;
        try {
            users = yield promisifiedListUsers.call(connection, search || '', Number.isInteger(limit) ? limit : -1, Number.isInteger(offset) ? offset : -1);
        }
        catch (error) {
            users = rejectWithOcsError(error, {
                message: 'Unable to list users',
                useMeta: false
            });
        }
        return users;
    });
}
export function deleteUser(connection, userId) {
    return __awaiter(this, void 0, void 0, function* () {
        let userDeleted;
        try {
            userDeleted = yield promisifiedDeleteUser.call(connection, userId);
        }
        catch (error) {
            userDeleted = rejectWithOcsError(error, {
                message: 'Unable to delete user',
                identifier: userId,
                useMeta: true,
                expectedErrorCodes: [400],
                customErrors: {
                    [101]: 'user does not exist'
                }
            });
        }
        return userDeleted;
    });
}
export function addUser(connection, user) {
    return __awaiter(this, void 0, void 0, function* () {
        let userAdded;
        try {
            userAdded = yield promisifiedAddUser.call(connection, user);
        }
        catch (error) {
            userAdded = rejectWithOcsError(error, {
                message: 'Unable to add user',
                identifier: (user && user.userid ? user.userid : ''),
                useMeta: true,
                expectedErrorCodes: [400],
                customErrors: {
                    [102]: 'username already exists',
                    [103]: 'unknown error occurred whilst adding the user',
                    [104]: 'group does not exist',
                    [105]: 'insufficient privileges for group',
                    [106]: 'no group specified (required for sub-admins',
                    [108]: 'password and email empty. Must set password or an email',
                    [109]: 'invitation email cannot be send'
                }
            });
        }
        return userAdded;
    });
}
export function listGroups(connection, search, limit, offset) {
    return __awaiter(this, void 0, void 0, function* () {
        let groups;
        try {
            groups = yield promisifiedListGroups.call(connection, search || '', Number.isInteger(limit) ? limit : -1, Number.isInteger(offset) ? offset : -1);
        }
        catch (error) {
            groups = rejectWithOcsError(error, {
                message: 'Unable to list groups',
                useMeta: false
            });
        }
        return groups;
    });
}
export function addGroup(connection, groupId) {
    return __awaiter(this, void 0, void 0, function* () {
        let groupAdded;
        try {
            groupAdded = yield promisifiedAddGroup.call(connection, groupId);
        }
        catch (error) {
            groupAdded = rejectWithOcsError(error, {
                message: 'Unable to add group',
                identifier: groupId,
                useMeta: true,
                expectedErrorCodes: [400],
                customErrors: {
                    [102]: 'group already exists',
                    [103]: 'failed to add the group'
                }
            });
        }
        return groupAdded;
    });
}
export function deleteGroup(connection, groupId) {
    return __awaiter(this, void 0, void 0, function* () {
        let groupDeleted;
        try {
            groupDeleted = yield promisifiedDeleteGroup.call(connection, groupId);
        }
        catch (error) {
            groupDeleted = rejectWithOcsError(error, {
                message: 'Unable to delete group',
                identifier: groupId,
                useMeta: true,
                expectedErrorCodes: [400],
                customErrors: {
                    [101]: 'group does not exist',
                    [102]: 'failed to delete group'
                }
            });
        }
        return groupDeleted;
    });
}
export function getGroupUsers(connection, groupId) {
    return __awaiter(this, void 0, void 0, function* () {
        let users;
        try {
            users = yield promisifiedGetGroupUsers.call(connection, groupId);
        }
        catch (error) {
            users = rejectWithOcsError(error, {
                message: 'Unable to list users for group',
                identifier: groupId,
                useMeta: false,
                expectedErrorCodes: [404],
                customErrors: {
                    [404]: 'the group could not be found'
                }
            });
        }
        return users;
    });
}
export function getGroupSubAdmins(connection, groupId) {
    return __awaiter(this, void 0, void 0, function* () {
        let subAdmins;
        try {
            subAdmins = yield promisifiedGetGroupSubAdmins.call(connection, groupId);
        }
        catch (error) {
            subAdmins = rejectWithOcsError(error, {
                message: 'Unable to list sub-admins for group',
                identifier: groupId,
                useMeta: true,
                expectedErrorCodes: [400],
                customErrors: {
                    [101]: 'group does not exist'
                }
            });
        }
        return subAdmins;
    });
}
export function getShares(connection, path, includeReshares, showForSubFiles) {
    return __awaiter(this, void 0, void 0, function* () {
        let shares;
        try {
            shares = yield promisifiedGetShares.call(connection, path || '', (includeReshares !== undefined ? includeReshares : false), (showForSubFiles !== undefined ? showForSubFiles : false));
        }
        catch (error) {
            shares = rejectWithOcsError(error, {
                message: 'Unable to get shares for',
                identifier: path,
                useMeta: true,
                expectedErrorCodes: [400, 404],
                customErrors: {
                    [400]: 'unable to show sub-files as this is not a directory',
                    [404]: 'file/folder doesn\'t exist'
                }
            });
        }
        return shares;
    });
}
export function getShare(connection, shareId) {
    return __awaiter(this, void 0, void 0, function* () {
        let share;
        try {
            share = yield promisifiedGetShare.call(connection, shareId);
        }
        catch (error) {
            share = rejectWithOcsError(error, {
                message: 'Unable to get share',
                identifier: shareId,
                useMeta: true,
                expectedErrorCodes: [404]
            });
        }
        return share;
    });
}
export function deleteShare(connection, shareId) {
    return __awaiter(this, void 0, void 0, function* () {
        let shareDeleted;
        try {
            shareDeleted = yield promisifiedDeleteShare.call(connection, shareId);
        }
        catch (error) {
            shareDeleted = rejectWithOcsError(error, {
                message: 'Unable to delete share',
                identifier: shareId,
                useMeta: true,
                expectedErrorCodes: [404],
                customErrors: {
                    [404]: 'invalid shareId or the share doesn\'t exist'
                }
            });
        }
        return shareDeleted;
    });
}
export function addShare(connection, path, shareType, shareWith, permissions, password, publicUpload) {
    return __awaiter(this, void 0, void 0, function* () {
        let addedShare;
        try {
            addedShare = yield promisifiedAddShare.call(connection, path, shareType, shareWith || '', (permissions !== undefined ? permissions : OcsSharePermissions.default), password || '', (publicUpload !== undefined ? publicUpload : false));
        }
        catch (error) {
            addedShare = rejectWithOcsError(error, {
                message: 'Unable to add share',
                identifier: path,
                useMeta: true,
                expectedErrorCodes: [403, 404]
            });
        }
        return addedShare;
    });
}
export function editShare(connection, shareId) {
    return {
        permissions(permissions) {
            return __awaiter(this, void 0, void 0, function* () {
                return yield setFieldValue(connection, shareId, 'permissions', permissions);
            });
        },
        password(password) {
            return __awaiter(this, void 0, void 0, function* () {
                return yield setFieldValue(connection, shareId, 'password', password);
            });
        },
        publicUpload(isPublicUpload) {
            return __awaiter(this, void 0, void 0, function* () {
                return yield setFieldValue(connection, shareId, 'publicUpload', isPublicUpload);
            });
        },
        expireDate(expireDate) {
            return __awaiter(this, void 0, void 0, function* () {
                return yield setFieldValue(connection, shareId, 'expireDate', expireDate);
            });
        },
        note(note) {
            return __awaiter(this, void 0, void 0, function* () {
                return yield setFieldValue(connection, shareId, 'note', note);
            });
        }
    };
    function setFieldValue(connection, shareId, field, value) {
        return __awaiter(this, void 0, void 0, function* () {
            let editedShare;
            try {
                editedShare = yield promisifiedEditShare.call(connection, shareId, field, String(value));
            }
            catch (error) {
                editedShare = rejectWithOcsError(error, {
                    message: `Unable to edit '${field}' of share`,
                    identifier: shareId,
                    useMeta: true,
                    expectedErrorCodes: [400, 404]
                });
            }
            return editedShare;
        });
    }
}
export function getGroupfolders(connection) {
    return __awaiter(this, void 0, void 0, function* () {
        let groupfolders;
        try {
            groupfolders = yield promisifiedGetGroupfolders.call(connection);
        }
        catch (error) {
            groupfolders = rejectWithOcsError(error, {
                message: 'Unable to list groupfolders',
                useMeta: true,
                expectedErrorCodes: [500],
            });
        }
        return groupfolders;
    });
}
export function getGroupfolder(connection, groupfolderId) {
    return __awaiter(this, void 0, void 0, function* () {
        let groupfolder;
        try {
            groupfolder = yield promisifiedGetGroupfolder.call(connection, groupfolderId);
        }
        catch (error) {
            groupfolder = rejectWithOcsError(error, {
                message: 'Unable to get groupfolder',
                identifier: groupfolderId,
                useMeta: true,
                expectedErrorCodes: [500],
            });
        }
        return groupfolder;
    });
}
export function addGroupfolder(connection, mountpoint) {
    return __awaiter(this, void 0, void 0, function* () {
        let addedGroupfolderId;
        try {
            addedGroupfolderId = yield promisifiedAddGroupfolder.call(connection, mountpoint);
        }
        catch (error) {
            addedGroupfolderId = rejectWithOcsError(error, {
                message: 'Unable to create groupfolder',
                identifier: mountpoint,
                useMeta: true,
                expectedErrorCodes: [500],
            });
        }
        return addedGroupfolderId;
    });
}
export function removeGroupfolder(connection, groupfolderId) {
    return __awaiter(this, void 0, void 0, function* () {
        let groupfolderDeleted;
        try {
            groupfolderDeleted = yield promisifiedRemoveGroupfolder.call(connection, groupfolderId);
        }
        catch (error) {
            groupfolderDeleted = rejectWithOcsError(error, {
                message: 'Unable to delete groupfolder',
                identifier: groupfolderId,
                useMeta: true,
                expectedErrorCodes: [500],
            });
        }
        return groupfolderDeleted;
    });
}
export function addGroupfolderGroup(connection, groupfolderId, groupId) {
    return __awaiter(this, void 0, void 0, function* () {
        let groupfolderGroupAdded;
        try {
            groupfolderGroupAdded = yield promisifiedAddGroupfolderGroup.call(connection, groupfolderId, groupId);
        }
        catch (error) {
            groupfolderGroupAdded = rejectWithOcsError(error, {
                message: 'Unable to add group to groupfolder',
                identifier: groupfolderId,
                useMeta: true,
                expectedErrorCodes: [500],
            });
        }
        return groupfolderGroupAdded;
    });
}
export function removeGroupfolderGroup(connection, groupfolderId, groupId) {
    return __awaiter(this, void 0, void 0, function* () {
        let groupfolderGroupRemoved;
        try {
            groupfolderGroupRemoved = yield promisifiedRemoveGroupfolderGroup.call(connection, groupfolderId, groupId);
        }
        catch (error) {
            groupfolderGroupRemoved = rejectWithOcsError(error, {
                message: 'Unable to remove group from groupfolder',
                identifier: groupfolderId,
                useMeta: true,
                expectedErrorCodes: [500],
            });
        }
        return groupfolderGroupRemoved;
    });
}
export function setGroupfolderPermissions(connection, groupfolderId, groupId, permissions) {
    return __awaiter(this, void 0, void 0, function* () {
        let groupfolderPermissionsSet;
        try {
            groupfolderPermissionsSet = yield promisifiedSetGroupfolderPermissions.call(connection, groupfolderId, groupId, permissions);
        }
        catch (error) {
            groupfolderPermissionsSet = rejectWithOcsError(error, {
                message: 'Unable to set groupfolder permissions',
                identifier: groupfolderId,
                useMeta: true,
                expectedErrorCodes: [500],
            });
        }
        return groupfolderPermissionsSet;
    });
}
export function enableGroupfolderACL(connection, groupfolderId, enable) {
    return __awaiter(this, void 0, void 0, function* () {
        let groupfolderACLEnabled;
        try {
            groupfolderACLEnabled = yield promisifiedEnableOrDisableGroupfolderACL.call(connection, groupfolderId, enable);
        }
        catch (error) {
            groupfolderACLEnabled = rejectWithOcsError(error, {
                message: 'Unable to enable ACL for groupfolder',
                identifier: groupfolderId,
                useMeta: true,
                expectedErrorCodes: [500],
            });
        }
        return groupfolderACLEnabled;
    });
}
export function setGroupfolderManageACL(connection, groupfolderId, type, id, manageACL) {
    return __awaiter(this, void 0, void 0, function* () {
        let groupfolderManageACLSet;
        try {
            groupfolderManageACLSet = yield promisifiedSetGroupfolderManageACL.call(connection, groupfolderId, type, id, manageACL);
        }
        catch (error) {
            groupfolderManageACLSet = rejectWithOcsError(error, {
                message: 'Unable to set groupfolder manage ACL settings',
                identifier: groupfolderId,
                useMeta: true,
                expectedErrorCodes: [500],
            });
        }
        return groupfolderManageACLSet;
    });
}
export function setGroupfolderQuota(connection, groupfolderId, quota) {
    return __awaiter(this, void 0, void 0, function* () {
        let groupfolderQuotaSet;
        try {
            groupfolderQuotaSet = yield promisifiedSetGroupfolderQuota.call(connection, groupfolderId, quota);
        }
        catch (error) {
            groupfolderQuotaSet = rejectWithOcsError(error, {
                message: 'Unable to set groupfolder quota',
                identifier: groupfolderId,
                useMeta: true,
                expectedErrorCodes: [500],
            });
        }
        return groupfolderQuotaSet;
    });
}
export function renameGroupfolder(connection, groupfolderId, mountpoint) {
    return __awaiter(this, void 0, void 0, function* () {
        let groupfolderRenamed;
        try {
            groupfolderRenamed = yield promisifiedRenameGroupfolder.call(connection, groupfolderId, mountpoint);
        }
        catch (error) {
            groupfolderRenamed = rejectWithOcsError(error, {
                message: 'Unable to rename groupfolder',
                identifier: groupfolderId,
                useMeta: true,
                expectedErrorCodes: [500],
            });
        }
        return groupfolderRenamed;
    });
}
