var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { NotFoundError, Exception as NextcloudError, ForbiddenError, ConflictError } from './errors';
const WRAPPED_FUNCTIONS = [
    ['copyFile', 0],
    ['createDirectory', 0],
    ['createReadStream', 0],
    ['createWriteStream', 0],
    ['customRequest', 0],
    ['deleteFile', 0],
    ['exists', 0],
    ['getDirectoryContents', 0],
    ['getFileContents', 0],
    ['getFileDownloadLink', 0],
    ['getFileUploadLink', 0],
    ['getHeaders', -1],
    ['getQuota', -1],
    ['lock', 0],
    ['moveFile', 0],
    ['putFileContents', 0],
    ['setHeaders', -1],
    ['stat', 0],
    ['unlock', 0]
];
export function wrapClient(client) {
    if (client['__ns_wrapped_client__']) {
        return client;
    }
    client['__ns_wrapped_client__'] = true;
    WRAPPED_FUNCTIONS.forEach(([fnName, pathPosition]) => {
        const originalFn = client[fnName];
        client[fnName] = (...args) => __awaiter(this, void 0, void 0, function* () {
            try {
                return yield originalFn.apply(client, args);
            }
            catch (error) {
                throw wrapError(error, pathPosition === -1 ? undefined : args[pathPosition]);
            }
        });
    });
    return client;
}
export function wrapError(error, path) {
    if (!isWebDavError(error)) {
        return error;
    }
    if (isNotFoundError(error)) {
        return NotFoundError(path);
    }
    if (isForbiddenError(error)) {
        return ForbiddenError(path);
    }
    if (isConflictError(error)) {
        return ConflictError(path);
    }
    return NextcloudError(error);
}
function isWebDavError(error) {
    return error && (error.response || error.status);
}
function isNotFoundError(error) {
    return error.status === 404;
}
function isForbiddenError(error) {
    return error.status === 403;
}
function isConflictError(error) {
    return error.status === 409;
}
